from ..enums import MeshLevel
from ..models import Indicator
from ..payloads import Territory
from .commons import (
    add_optional_filters,
    calculate_aggregate_values,
    get_last_year,
    get_mesh_column_name,
    get_where_territory,
)
from .utils import get_breakdown_dimension, run_custom_query


def get_proportions_chart(indicator, territory, filters):
    where_territory = get_where_territory(territory)
    last_year = get_last_year(indicator, territory.mesh)
    breakdown_dimension = get_breakdown_dimension(indicator)

    query = f"""
    SELECT {calculate_aggregate_values(indicator, with_alternative=False)}, "{breakdown_dimension.db_name}" as dimension
    FROM "{indicator.db_table_prefix}_{territory.mesh}" as indic
    WHERE {where_territory} AND annee = {last_year}
    {add_optional_filters(indicator, filters)}
    GROUP BY "{breakdown_dimension.db_name}"
"""
    data = run_custom_query(query)
    filters_color = {f.db_name: f.color for f in breakdown_dimension.filters.all()}
    data_dict = {
        d["dimension"]: {
            "label": d["dimension"],
            "data": [d["valeur"]],
            "color": filters_color.get(d["dimension"]),
        }
        for d in data
    }
    breakdown_filters = filters[breakdown_dimension.db_name]
    sorted_data = (
        [data_dict[filter] for filter in breakdown_filters if filter in data_dict]
        if filters
        else list(data_dict.values())
    )
    return sorted_data


def get_values_for_submesh_territories(
    indicator: Indicator, submesh: MeshLevel, territory: Territory, filters
):
    submesh = submesh.lower()
    mesh_col = get_mesh_column_name(submesh)
    parent_territory_join = f"JOIN arbo_{territory.mesh}_{submesh} j ON j.{submesh} = arbo.code AND j.{territory.mesh} IN {territory.sql_codes}"
    if submesh == territory.mesh:
        parent_territory_join = f"AND arbo.code IN {territory.sql_codes}"
    query = f"""
    SELECT {calculate_aggregate_values(indicator)}, indic.{mesh_col} as geocode, arbo.name as geoname
    FROM "{indicator.db_table_prefix}_{submesh}" indic
    JOIN arbo_{submesh} arbo ON arbo.code = indic.{mesh_col}
    {parent_territory_join}
    {add_optional_filters(indicator, filters)}
    AND annee = (
        SELECT MAX(annee)
        FROM "{indicator.db_table_prefix}_{submesh}"
    )
    GROUP BY geocode, geoname
    """
    return run_custom_query(query)



# Generated by Django 5.2.7 on 2025-10-13 12:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('indicators_lib', '0002_filter_color_indicator_min_mesh_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='indicator',
            name='short_title',
            field=models.CharField(blank=True, max_length=128, null=True),
        ),
    ]



# Generated by Django 5.2.7 on 2025-10-28 12:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geo_lib', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='geoelement',
            name='linked_to_indicator',
            field=models.BooleanField(default=False, help_text="Si coché, le filtre de l'indicateur avec le même nom sere relié au filtre de la carte.", verbose_name="Relié aux filtres de l'indicateur"),
        ),
    ]



from django.conf import settings

from territories_dashboard_lib.superset_lib.models import Dashboard
from territories_dashboard_lib.superset_lib.serializers import serialize_dashboard
from territories_dashboard_lib.website_lib.conf import get_main_conf
from territories_dashboard_lib.website_lib.models import NoticeBanner


def default(request):
    main_conf = get_main_conf()
    notice = NoticeBanner.objects.first()
    view_name = request.resolver_match.view_name if request.resolver_match else None
    context = {
        "ENABLE_SUPERSET": settings.ENABLE_SUPERSET,
        "ANALYTICS_ID": settings.ANALYTICS_ID,
        "ENVIRONMENT": settings.ENVIRONMENT,
        "view_name": view_name,
        "absolute_uri": request.build_absolute_uri(),
        "main_conf": main_conf,
        "notice": notice,
        "dashboards": [
            serialize_dashboard(d)
            for d in Dashboard.objects.all().order_by("order", "label")
        ],
        "version": settings.VERSION,
    }
    return context



from django.db import models
from martor.models import MartorField

from territories_dashboard_lib.commons.models import CommonModel
from territories_dashboard_lib.indicators_lib.enums import (
    AggregationFunctions,
    MeshLevel,
)
from territories_dashboard_lib.indicators_lib.refresh_filters import refresh_filters


class Theme(CommonModel):
    ordering = models.IntegerField(default=0)
    ordering.verbose_name = "Ordre dans la sidebar"
    name = models.CharField(max_length=64, unique=True)
    name.verbose_name = "Nom (~id, URL)"
    title = models.CharField(max_length=128)
    title.verbose_name = "Titre (affiché)"
    objectif_theme = models.TextField(blank=True)
    objectif_theme.verbose_name = "Objectif"
    action_theme = models.TextField(blank=True)
    action_theme.verbose_name = "Actions"

    def is_displayed_on_app(self):
        return self.sub_themes.all().exists()

    is_displayed_on_app.__name__ = "Visible ?"
    is_displayed_on_app.boolean = True

    def subthemes_count(self):
        return self.sub_themes.count()

    subthemes_count.__name__ = "Nb sous-thèmes"

    def indicators_count(self):
        return sum(
            [sub_theme.indicators_count() for sub_theme in self.sub_themes.all()]
        )

    indicators_count.__name__ = "Total indicateurs"

    def __str__(self):
        return self.title

    class Meta:
        ordering = ("ordering",)
        verbose_name = "1 - Thème"
        indexes = [
            models.Index(fields=["name"]),
        ]


class SubTheme(CommonModel):
    name = models.CharField(max_length=64, unique=True)
    name.verbose_name = "Nom (~id, URL)"
    theme = models.ForeignKey(
        Theme, on_delete=models.CASCADE, related_name="sub_themes"
    )
    theme.verbose_name = "Thème"
    index_in_theme = models.IntegerField(default=0)
    index_in_theme.verbose_name = "Ordre thème"
    title = models.CharField(max_length=128)
    title.verbose_name = "Titre (affiché)"
    description = models.TextField(default="", blank=True)

    def __str__(self):
        return f"{self.theme.title} > {self.title}"

    def indicators_count(self):
        return self.indicators.count()

    def is_displayed_on_app(self):
        return self.indicators.all().exists()

    indicators_count.__name__ = "Nb indicateurs"
    is_displayed_on_app.__name__ = "Visible ?"
    is_displayed_on_app.boolean = True

    class Meta:
        ordering = ("theme", "index_in_theme")
        verbose_name = "2 - Sous-thème"
        indexes = [
            models.Index(fields=["theme", "index_in_theme"]),
            models.Index(fields=["name"]),
        ]


class Indicator(CommonModel):
    is_active = models.BooleanField(default=True)
    is_active.verbose_name = "Actif ?"
    sub_theme = models.ForeignKey(
        SubTheme, on_delete=models.CASCADE, related_name="indicators"
    )
    sub_theme.verbose_name = "Sous-thème"
    index_in_theme = models.IntegerField(default=0)  # TODO: is it necessary?
    index_in_theme.verbose_name = "Ordre sous-thème"
    name = models.CharField(max_length=32, unique=True)
    name.verbose_name = "Nom (~id, URL)"
    title = models.CharField(max_length=128)
    title.verbose_name = "Titre (affiché)"
    short_title = models.CharField(max_length=128, null=True, blank=True)
    short_title.verbose_name = "Titre abrégé"
    # Indicator's DB attributes
    db_table_prefix = models.CharField(max_length=128)
    db_table_prefix.verbose_name = "Préfixe dans la DB"
    min_mesh = models.TextField(choices=MeshLevel.choices, default=MeshLevel.com)
    is_composite = models.BooleanField(default=False)
    is_composite.verbose_name = "Indicateur composite"
    show_alternative = models.BooleanField(
        default=True,
        verbose_name="Afficher la valeur alternative",
        help_text="Pour certains indicateurs composites (par exemple les moyennes) on ne veut pas afficher la valeur alternative.",
    )
    aggregation_constant = models.DecimalField(
        default=1, decimal_places=5, max_digits=10
    )
    aggregation_constant.verbose_name = "Constante d'agrégation"
    aggregation_function = models.CharField(
        default=AggregationFunctions.REPEATED_COMPONENT_2,
        max_length=8,
        choices=AggregationFunctions.choices,
    )
    aggregation_function.verbose_name = "Fonction d'agrégation"
    unite = models.CharField(max_length=32)
    unite.verbose_name = "Unité (affichée)"
    unite_nom_accessible = models.CharField(
        max_length=64,
        default="",
        blank=True,
        help_text="Nom accessible de l'unité qui sera lu par le lecteur d'écran.",
    )
    unite_alternative = models.CharField(
        default=None, null=True, blank=True, max_length=32
    )
    unite_alternative.verbose_name = "Unité alternative (affichée)"
    unite_alternative_nom_accessible = models.CharField(
        max_length=64,
        default="",
        blank=True,
        help_text="Nom accessible de l'unité alternative qui sera lu par le lecteur d'écran.",
    )
    secondary_indicator = models.ForeignKey(
        "indicators_lib.Indicator",
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        verbose_name="Second indicateur à afficher sur la carte",
    )
    flows_db_table_prefix = models.CharField(
        null=True,
        blank=True,
        max_length=128,
        verbose_name="Table des flux (prefix)",
        help_text="Les données des flux seront affichés sur la carte et le graphique de Sankey.",
    )
    flows_dimension = models.CharField(
        null=True,
        blank=True,
        max_length=32,
        verbose_name="Dimension des flux",
        help_text="Nom de la colonne des dimensions de la table des flux",
    )
    # Descriptive attributes
    show_evolution = models.BooleanField(default=True)
    show_evolution.verbose_name = "Activer l'historique"
    source = models.TextField(default="", blank=True)
    description = models.TextField(default="", blank=True)
    methodo = MartorField(default="", blank=True)
    methodo.verbose_name = "Méthodologie (markdown)"
    methodo_file = models.BinaryField(null=True, blank=True)

    def __str__(self):
        return self.title

    def get_theme_title(self):
        return self.sub_theme.theme.title

    get_theme_title.short_description = "Thème"

    def table_name(indicator, mesh: MeshLevel, *, flows: bool = False):
        table_prefix = (
            indicator.flows_db_table_prefix if flows else indicator.db_table_prefix
        )
        return f"{table_prefix}_{mesh}"

    class Meta:
        ordering = ("sub_theme", "index_in_theme")
        verbose_name = "3 - Indicateur"
        indexes = [
            models.Index(fields=["sub_theme"]),
            models.Index(fields=["name"]),
        ]


class Dimension(CommonModel):
    indicator = models.ForeignKey(
        Indicator, on_delete=models.CASCADE, related_name="dimensions"
    )
    db_name = models.TextField()
    title = models.TextField()
    is_breakdown = models.BooleanField(
        default=False,
        verbose_name="Répartir selon cette dimension",
        help_text="Dans le cas de plusieurs dimensions pour un indicateur, l'une d'entre elles doit être la dimension de réparition pour les graphiques.",
    )

    def save(self, *args, **kwargs):
        is_new = self.pk is None
        result = super().save(*args, **kwargs)
        if is_new:
            try:
                refresh_filters(self)
            except Exception as e:
                print(e)
        return result


class Filter(CommonModel):
    dimension = models.ForeignKey(
        Dimension, related_name="filters", on_delete=models.CASCADE
    )
    db_name = models.CharField(max_length=128)
    db_name.verbose_name = "Nom dans la BDD"
    order = models.IntegerField(default=0)
    order.verbose_name = "Ordre"
    default = models.BooleanField(default=True)
    default.verbose_name = "Sélectionné par défaut ?"
    color = models.TextField(
        null=True,
        blank=True,
        verbose_name="Couleur",
        help_text="format hexadécimale : #FF11DD",
    )

    def __str__(self):
        return self.db_name

    class Meta:
        ordering = ("dimension", "order", "db_name")
        verbose_name = "Filtre"
        indexes = [
            models.Index(fields=["db_name"]),
            models.Index(fields=["dimension", "order"]),
        ]
        unique_together = ("dimension", "db_name")



import functools
from typing import get_origin

from django.http import HttpResponse
from pydantic import ValidationError


def use_payload(payload_class):
    def decorator(func):
        @functools.wraps(func)
        def wrapper(request, *args, **kwargs):
            if request.method != "GET":
                raise NotImplementedError
            data = {}
            for field_name, field in payload_class.model_fields.items():
                url_name = field.alias if field.alias else field_name
                is_list_field = get_origin(field.annotation) is list
                url_value = (
                    request.GET.getlist(url_name)
                    if is_list_field
                    else request.GET.get(url_name)
                )
                if url_value is not None:
                    data[url_name] = url_value
            try:
                payload = payload_class.model_validate(data)
            except ValidationError as e:
                return HttpResponse(
                    e.json(), headers={"content_type": "application/json"}, status=400
                )

            return func(request, *args, payload=payload, **kwargs)

        return wrapper

    return decorator



from django.contrib.postgres.fields import ArrayField
from django.db import models
from martor.models import MartorField

from territories_dashboard_lib.commons.models import CommonModel
from territories_dashboard_lib.indicators_lib.enums import STANDARD_MESHES, MeshLevel
from territories_dashboard_lib.website_lib.navigation import (
    parse_footer_navigation,
    parse_header_navigation,
    parse_markdown_link,
)
from territories_dashboard_lib.website_lib.static_content import (
    markdown_content_to_html,
)


def get_default_meshes():
    return STANDARD_MESHES


class MainConf(CommonModel):
    title = models.TextField(
        verbose_name="Titre principal du site", default="Tableau de bord"
    )
    description = models.TextField(
        verbose_name="Description du site",
        null=True,
        blank=True,
        help_text="Utilisé pour la balise meta description du site et pour le relais sur les réseaux sociaux.",
    )
    entity = models.TextField(
        verbose_name="Entité qui possède le site",
        default="",
        help_text="Renseigner le nom de l'entité au même format que le logo officiel en respectant les mises à la ligne.",
    )
    header_navigation = models.TextField(
        verbose_name="Navigation du header",
        help_text="""Navigation du header, mettre les liens au format markdown avec un slash à la fin du lien, mettre une ligne vide entre chaque rubrique. <br/><br/>
        
        Une rubrique peut contenir un seul lien, ou bien plusiers, dans ce cas la rubrique sera sous forme de dropdown et il faut mettre un titre en premier. <br/><br/>Par exemple : <br/>
        <br/><br/>
        [Accueil](/accueil/)
        <br/><br/>
        [Indicateurs territoriaux](/indicateurs/)
        <br/><br/>
        À propos
        [Présentation](/presentation/)
        [Journal des versions](/journal/)
        """,
        default="",
    )
    footer_navigation = models.TextField(
        verbose_name="Navigation du footer",
        help_text="""Navigation du footer, mettre les liens au format markdown, un par ligne. <br/><br/>
    Si le lien est interne, commencer et terminer le lien par un slash comme "/accueil/"<br/><br/>
    Par exemple : <br/><br/>
    [Plan du site](/plan-site/)<br/>
    [Accessibilité](/accessibilite/)<br/>
    [Mentions légales](/mentions-legales/) <br/>
    """,
        default="",
    )
    contact_email = models.TextField(
        default=None, null=True, blank=True, verbose_name="Email de contact"
    )
    newsletter_link = models.TextField(
        default=None,
        null=True,
        blank=True,
        verbose_name="Lien d'inscription à la newsletter",
    )
    show_footer_contact_banner = models.BooleanField(
        default=False,
        verbose_name="Afficher une bannière de contact au-dessus du footer",
        help_text="Pour afficher la bannière, l'email de contact et le lien d'inscription à la newsletter doivent être renseignés.",
    )
    social_image_url = models.TextField(
        null=True,
        blank=True,
        verbose_name="Image de description",
        help_text="Lien d'une image (idéalement 1200x630px). Elle sera utilisée lors du partage sur les réseaux sociaux.",
    )
    meshes = ArrayField(
        models.TextField(choices=MeshLevel),
        default=get_default_meshes,
        verbose_name="Mailles",
        help_text=f"Liste des mailles territoriales activées sur la plateforme, la liste des mailles disponible est : {[m.value for m in MeshLevel]}.<br/>L'ajout de nouvelles mailles demande une modification de la base de données des indicateurs ainsi que du code.",
    )

    @property
    def entity_breaklines(self):
        return self.entity.replace("\n", "<br/>")

    @property
    def parsed_header_navigation(self):
        return parse_header_navigation(self.header_navigation)

    @property
    def parsed_footer_navigation(self):
        return parse_footer_navigation(self.footer_navigation)

    def __str__(self):
        return "Configuration principale"

    class Meta:
        verbose_name = "Configuration principale"
        verbose_name_plural = "Configurations principales"


class LandingPage(CommonModel):
    title = models.TextField(verbose_name="Titre")
    body = MartorField(verbose_name="Contenu")
    button_link = models.TextField(
        verbose_name="Lien du bouton",
        help_text="Au format markdown, par exemple : <br/><br/>[découvrir les indicateurs](/indicateurs/)",
    )

    @property
    def body_html(self):
        return markdown_content_to_html(self.body)

    @property
    def button(self):
        return parse_markdown_link(self.button_link)

    class Meta:
        verbose_name = "Page d'accueil"
        verbose_name_plural = "Page d'accueil"

    def __str__(self):
        return "Page d'accueil"


class StaticPage(CommonModel):
    name = models.TextField(verbose_name="Nom de la page")
    url = models.TextField(
        verbose_name="Url de la page",
        help_text="En minuscule avec des espaces remplacés par des traits '-'.<br/>Par exemple : <br/><br/> indicateurs <br/> mentions-legales",
    )
    body = MartorField(verbose_name="Contenu")

    @property
    def body_html(self):
        return markdown_content_to_html(self.body)

    class Meta:
        verbose_name = "Page Statique"
        verbose_name_plural = "Pages Statiques"

    def __str__(self):
        return self.name


class GlossaryItem(CommonModel):
    word = models.TextField(verbose_name="Mot")
    definition = models.TextField(verbose_name="Définition")

    class Meta:
        verbose_name = "Définition"
        verbose_name_plural = "Page Lexique"
        ordering = ["word"]

    def __str__(self):
        return self.word


class NoticeBanner(CommonModel):
    title = models.CharField(max_length=64, verbose_name="Titre")
    description = models.CharField(max_length=255, verbose_name="Description")
    link_href = models.CharField(
        null=True,
        blank=True,
        max_length=255,
        verbose_name="Lien",
        help_text="Le lien doit être un lien interne et commencer et terminer par '/', par exemple '/journal/",
    )
    link_text = models.CharField(
        null=True,
        blank=True,
        max_length=64,
        verbose_name="Intitulé du lien",
        help_text="à remplir seulement si un lien est spécifié",
    )

    class Meta:
        ordering = ["-created_at"]



from django.conf import settings
from django.db import connections


def format_sql_codes(codes: list[str]):
    """
    Format list of IDs for SQL IN statement.
    ["1","2","3"] => ('1', '2', '3')
    """
    codes = ", ".join(f"'{code.strip()}'" for code in codes)
    return f"({codes})"


def get_breakdown_dimension(indicator):
    breakdown_dimension = (
        indicator.dimensions.filter(is_breakdown=True).first()
        if indicator.dimensions.count() > 1
        else indicator.dimensions.first()
    )
    return breakdown_dimension


def get_connection():
    """
    The connection to the indicators database is defined in DATABASES
    The key for the database in DATABASES is defined in settings.INDICATORS_DATABASE
    During testing, the connection can also be defined in READONLY_DATABASES_HANDLER
    which defines a readonly database which will not be recreated during tests
    """
    connection = None
    indicators_database = getattr(settings, "INDICATORS_DATABASE", None)
    if indicators_database:
        connection = connections[indicators_database]
    else:
        testing = getattr(settings, "TESTING", False)
        readonly_databases_handler = getattr(
            settings, "READONLY_DATABASES_HANDLER", None
        )
        if testing and readonly_databases_handler:
            connection = readonly_databases_handler["default"]
    if connection is None:
        raise ValueError("No connection was defined for the indicators database")
    return connection


def run_custom_query(query, params=None):
    connection = get_connection()
    with connection.cursor() as cursor:
        cursor.execute(query, params)
        columns = [col[0] for col in cursor.description]
        rows = cursor.fetchall()
        results = [dict(zip(columns, row)) for row in rows]
        return results



# Generated by Django 5.2.3 on 2025-06-30 16:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tracking_lib', '0003_cookieinfo'),
    ]

    operations = [
        migrations.AddField(
            model_name='cookieinfo',
            name='ip_address',
            field=models.TextField(blank=True, null=True),
        ),
    ]



from django.urls import path

from .views import dashboards_view, guest_token_view

app_name = "superset-api"

urlpatterns = [
    path("guest-token/", guest_token_view, name="guest-token"),
    path("dashboards/", dashboards_view, name="dashboards"),
]



from territories_dashboard_lib.indicators_lib.payloads import Territory

from ..enums import FRANCE_DB_VALUES, MeshLevel
from ..models import AggregationFunctions, Indicator
from .utils import format_sql_codes, get_breakdown_dimension, run_custom_query


def get_mesh_column_name(mesh: MeshLevel):
    return f"code_{mesh}"


def get_sub_territories(
    *,
    submesh: MeshLevel,
    territory: Territory = None,
    with_center=False,
    codes: list[str] = None,
):
    """
    Récupère une liste des territoires appartenant à un territoire parent.
    params:
        submesh: maille des sous-territoires à récupérer
        territory: territoire parent des sous-territoires
        with_center: si True, ajoute le centre de chaque territoire
        codes: codes des sous-territoires à récupérer, peut-être utilisé en absence du territoire parent
    """
    parent_mesh_join = ""
    condition = ""
    if territory is None and codes:
        condition = f"WHERE arbo.code IN {format_sql_codes(codes)}"
    elif territory and submesh == territory.mesh:
        condition = f"WHERE arbo.code IN {territory.sql_codes}"
    elif territory:
        parent_mesh_join = f"JOIN arbo_{territory.mesh}_{submesh} AS j ON j.{submesh}=arbo.code AND j.{territory.mesh} IN {territory.sql_codes}"

    center = ""
    contours_join = ""
    if with_center:
        center = ", ST_ASGEOJSON(ST_CENTROID(contours.geometry)) as center"
        contours_join = f"JOIN contours_simplified_{submesh} as contours ON arbo.code = contours.code"

    query = f"""
        SELECT DISTINCT
            arbo.code,
            arbo.name
            {center}
        FROM arbo_{submesh} AS arbo
        {parent_mesh_join}
        {contours_join}
        {condition}
    """
    return run_custom_query(query)


def get_last_year(indicator, mesh):
    query = f""" SELECT MAX(annee) as last_year FROM "{indicator.db_table_prefix}_{mesh}" """
    return run_custom_query(query)[0]["last_year"]


def generate_aggregate_query(indicator, territory, submesh, filters, slicer):
    query = f"""
    WITH annee_max AS
    (SELECT MAX(annee) FROM "{indicator.db_table_prefix}_{submesh}"),
    aggregat AS
    (
        SELECT
        {calculate_aggregate_values(indicator)}, indic.{slicer}
        FROM
        {get_values_for_indicator_territory_submesh(indicator=indicator, territory=territory, submesh=submesh)}
        {add_optional_filters(indicator, filters)}
        AND annee = (SELECT * FROM annee_max)
        GROUP BY (indic.{slicer})
    )
    """
    return query


def generate_aggregate_query_for_location(indicator, territory, submesh, filters):
    mesh_col = get_mesh_column_name(submesh)
    return generate_aggregate_query(indicator, territory, submesh, filters, mesh_col)


def add_optional_filters(indicator: Indicator, filters):
    condition = ""
    all_dimensions = [dimension.db_name for dimension in indicator.dimensions.all()]
    for dimension in all_dimensions:
        if filters and filters.get(dimension):
            filters_str = ", ".join(
                [f"'{value.replace("'", "''")}'" for value in filters.get(dimension)]
            )
            condition += f' AND indic."{dimension}" in ({filters_str}) '
    return condition


def get_values_for_indicator_territory_submesh(
    *,
    indicator: Indicator,
    territory: Territory,
    submesh: MeshLevel,
    flows: bool = False,
):
    mesh_col = get_mesh_column_name(submesh)
    if submesh == territory.mesh:
        arbo_join = f"""
        JOIN arbo_{submesh} arbo ON arbo.code = indic.{mesh_col} AND arbo.code IN {territory.sql_codes}
        """
    else:
        arbo_join = f"""
        JOIN arbo_{territory.mesh}_{submesh} j 
            ON j.{submesh} = indic.{mesh_col} AND j.{territory.mesh} IN {territory.sql_codes}
        JOIN arbo_{submesh} arbo
            ON arbo.code = indic.{mesh_col}
        """
    if flows:
        arbo_join = f"""
        JOIN arbo_{territory.mesh}_{submesh} j1 
            ON j1.{submesh} = indic.{mesh_col}_1 AND j1.{territory.mesh} IN {territory.sql_codes}
        JOIN arbo_{submesh} arbo1
            ON arbo1.code = indic.{mesh_col}_1

        JOIN arbo_{territory.mesh}_{submesh} j2
            ON j2.{submesh} = indic.{mesh_col}_2 AND j2.{territory.mesh} IN {territory.sql_codes}
        JOIN arbo_{submesh} arbo2
            ON arbo2.code = indic.{mesh_col}_2
        """

    return f"""
    "{indicator.table_name(submesh, flows=flows)}" indic
    {arbo_join}
    """


def calculate_aggregate_values(indicator, with_alternative=True):
    if not indicator.is_composite:
        return "SUM(valeur) as valeur"

    if indicator.aggregation_function == AggregationFunctions.DISCRETE_COMPONENT_2:
        sql = f"SUM(composante_1) / COALESCE(NULLIF(SUM(composante_2), 0), 1) * {indicator.aggregation_constant} as valeur"
        if with_alternative:
            sql += ", SUM(composante_1) as valeur_alternative"
        return sql
    breakdown_dimension = get_breakdown_dimension(indicator)
    breakdown_count = (
        f" * COUNT(DISTINCT({breakdown_dimension.db_name})) "
        if breakdown_dimension
        else ""
    )
    sql = f"SUM(composante_1) / COALESCE(NULLIF(SUM(composante_2), 0), 1) {breakdown_count} * {indicator.aggregation_constant} as valeur"
    if with_alternative:
        sql += ", SUM(composante_1) as valeur_alternative"
    return sql


def get_where_territory(territory):
    territory_id = (
        FRANCE_DB_VALUES[territory.id]
        if territory.mesh == MeshLevel.fr
        else territory.id
    )
    mesh_col = get_mesh_column_name(territory.mesh)
    return f""" "{mesh_col}" = '{territory_id}' """


def get_values_for_territory(indicator, territory, filters=None):
    value = calculate_aggregate_values(indicator)
    where_territory = get_where_territory(territory)
    query = f"""
        SELECT {value}, annee
        FROM "{indicator.db_table_prefix}_{territory.mesh}" as indic
        WHERE {where_territory}
        {add_optional_filters(indicator, filters)}
        GROUP BY annee
        ORDER BY annee DESC
    """
    return query


def get_territory_name(territory):
    query = f"""SELECT name FROM arbo_{territory.mesh} WHERE code IN {territory.sql_codes} LIMIT 1;"""
    results = run_custom_query(query)
    return results[0]["name"] if results else ""



from django.db import models
from django.utils import timezone

from territories_dashboard_lib.indicators_lib.enums import MeshLevel
from territories_dashboard_lib.tracking_lib.enums import EventType


class Page(models.Model):
    """
    Une vue correspond à l'ouverture d'une des pages de l'application.
    À chaque changement de paramètre (territoire, maille, territoire de comparaison) une nouvelle vue est enregistrée.
    """

    created_at = models.DateTimeField(default=timezone.now)
    cookie = models.TextField(
        help_text="Un cookie sans durée d'expiration est placé dans chaque nouvelle session, il possède une valeur aléatoire qui permet de tracer le navigateur du visiteur entre plusieurs visites."
    )
    territory_id = models.TextField(
        help_text="ID du territoire principal sélectionné.", null=True, blank=True
    )
    territory_mesh = models.TextField(
        choices=MeshLevel.choices,
        help_text="Maille du territoire principal sélectionné.",
        null=True,
        blank=True,
    )
    submesh = models.TextField(
        choices=MeshLevel.choices,
        help_text="Maille d'analyse sélectionnée.",
        null=True,
        blank=True,
    )
    cmp_territory_id = models.TextField(
        null=True,
        blank=True,
        help_text="ID du territoire de comparaison, null s'il ne s'agit pas de la page comparaison.",
    )
    cmp_territory_mesh = models.TextField(
        choices=MeshLevel.choices,
        null=True,
        blank=True,
        help_text="Maille du territoire de comparaison, null s'il ne s'agit pas de la page comparaison.",
    )
    indicator = models.IntegerField(
        null=True,
        blank=True,
        help_text="ID de l'indicateur de la page détails, null s'il ne s'agit pas de la page détails, utile pour faire des jointures.",
    )
    indicator_name = models.TextField(
        null=True,
        blank=True,
        help_text="Nom de l'indicateur de la page détails, null s'il ne s'agit pas de la page détails, utile pour une analyse rapide.",
    )
    theme = models.IntegerField(
        help_text="ID du thème de la page, utile pour faire des jointures.",
        null=True,
        blank=True,
    )
    theme_name = models.TextField(
        help_text="Nom du thème de la page, utile pour une analyse rapide.",
        null=True,
        blank=True,
    )
    url = models.TextField(help_text="URL brut de la page")
    view = models.TextField(help_text="Nom de la vue dans Django")


class Event(models.Model):
    """
    Tracking des autres événements de l'application
    """

    created_at = models.DateTimeField(default=timezone.now)
    cookie = models.TextField(
        help_text="Un cookie sans durée d'expiration est placé dans chaque nouvelle session, il possède une valeur aléatoire qui permet de tracer le navigateur du visiteur entre plusieurs visites."
    )
    name = models.TextField(choices=EventType.choices)
    data = models.JSONField(null=True, blank=True)
    url = models.TextField(help_text="URL brut de la page")
    view = models.TextField(help_text="Nom de la vue dans Django")


class CookieInfo(models.Model):
    id = models.TextField(primary_key=True)
    country = models.TextField(null=True, blank=True)
    ip_address = models.TextField(null=True, blank=True)



from territories_dashboard_lib.indicators_lib.enums import (
    ALL_MESHES_ABSOLUTE,
    MESHES_ORDERED_FOR_PRESENTATION,
)
from territories_dashboard_lib.website_lib.models import MainConf


class MissingMainConf(Exception):
    def __init__(self):
        super().__init__()
        self.message = "Configuration principale du site (MainConf) manquante, veuillez la créer via le backoffice ou le shell."


def get_meshes_for_current_project():
    main_conf = get_main_conf()
    return [m for m in ALL_MESHES_ABSOLUTE if m in main_conf.meshes]


def get_ordered_meshes_for_current_project():
    main_conf = get_main_conf()
    return [m for m in MESHES_ORDERED_FOR_PRESENTATION if m in main_conf.meshes]


def get_main_conf():
    main_conf = MainConf.objects.first()
    if main_conf is None:
        raise MissingMainConf
    return main_conf



from django import template
from django.urls import reverse

from territories_dashboard_lib.website_lib.conf import get_meshes_for_current_project

register = template.Library()


@register.filter
def get_item(dictionary, key):
    return dictionary.get(key)


@register.filter
def indicator_api_urls(indicator_dict):
    view_names = [
        "download-indicator-methodo",
        "statistics",
        "values",
        "histogram",
        "top-10",
        "details-table",
        "details-table-export",
        "comparison-histogram",
        "proportions",
    ]
    return {
        view_name: reverse(
            f"indicators-api:{view_name}", kwargs={"name": indicator_dict["name"]}
        )
        for view_name in view_names
    }


@register.filter
def should_mesh_analysis(params, indicator):
    all_meshes = get_meshes_for_current_project()
    above_min_mesh = all_meshes.index(params["mesh"]) <= all_meshes.index(
        indicator["min_mesh"]
    )
    return above_min_mesh



from django.contrib import admin
from django.db import models
from django.forms import TextInput

from .models import Dashboard, Filter


class FilterInLine(admin.TabularInline):
    model = Filter
    extra = 0
    formfield_overrides = {
        models.TextField: {"widget": TextInput(attrs={"size": "32"})},
    }


@admin.register(Dashboard)
class DashboardAdmin(admin.ModelAdmin):
    formfield_overrides = {
        models.TextField: {"widget": TextInput(attrs={"size": "32"})},
    }
    inlines = [FilterInLine]
    list_display = ["short_name", "order"]



from typing import Optional

from pydantic import BaseModel

from territories_dashboard_lib.tracking_lib.enums import EventType


class EventPayload(BaseModel):
    indicator: str
    event: EventType
    objet: Optional[str] = None
    type: Optional[str] = None



# Generated by Django 5.2.8 on 2025-12-03 13:42

import django.contrib.postgres.fields
import territories_dashboard_lib.website_lib.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('website_lib', '0004_mainconf_description_mainconf_social_image_url'),
    ]

    operations = [
        migrations.AddField(
            model_name='mainconf',
            name='meshes',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.TextField(choices=[('fr', 'Fr'), ('reg', 'Reg'), ('dep', 'Dep'), ('epci', 'Epci'), ('com', 'Com'), ('aom', 'Aom')]), default=territories_dashboard_lib.website_lib.models.get_default_meshes, help_text="Liste des mailles territoriales activées sur la plateforme, la liste des mailles disponible est : ['fr', 'reg', 'dep', 'epci', 'com', 'aom'].<br/>L'ajout de nouvelles mailles demande une modification de la base de données des indicateurs ainsi que du code.", size=None, verbose_name='Mailles'),
        ),
    ]



from typing import Annotated, Optional

from pydantic import BaseModel, BeforeValidator, Field, model_validator

from territories_dashboard_lib.commons.types import MutualisedTerritoryCode, SQlName
from territories_dashboard_lib.indicators_lib.query.utils import format_sql_codes

from .enums import MeshLevel


class Territory(BaseModel):
    id: MutualisedTerritoryCode
    mesh: MeshLevel

    @property
    def sql_codes(territory) -> str:
        return format_sql_codes(territory.id.split(","))


def validate_territory(value):
    if value and "-" in value:
        return {"id": value.split("-")[0], "mesh": value.split("-")[1]}


class BasePayload(BaseModel):
    territory: Annotated[Territory, BeforeValidator(validate_territory)]


class SubMeshPayload(BasePayload):
    submesh: MeshLevel


class SubMeshOnlyPayload(SubMeshPayload):
    @model_validator(mode="after")
    def check_not_same_mesh(self):
        if self.submesh == self.territory.mesh:
            raise ValueError("submesh and territory mesh should not be equal")
        return self


class FlowsPayload(SubMeshPayload):
    prefix: SQlName
    dimension: SQlName | None = None


class ComparisonQueryPayload(SubMeshPayload):
    cmp_territory: Annotated[
        Territory, BeforeValidator(validate_territory), Field(alias="cmp-territory")
    ]


class OptionalComparisonQueryPayload(SubMeshPayload):
    cmp_territory: Annotated[
        Optional[Territory],
        BeforeValidator(validate_territory),
        Field(default=None, alias="cmp-territory"),
    ]


class IndicatorTablePayload(SubMeshPayload):
    column_order: SQlName | None = None
    column_order_flow: SQlName | None = None
    pagination: int = 1
    limit: int = 20
    previous_limit: int | None = None
    search: str | None = None
    year: int | None = None
    flows: bool | None = False
    focus: bool | None = False



def _get_precision(value, force_integer):
    return f"{value:.0f}" if force_integer else f"{value:.1f}"


def _remove_useless_0(value):
    if "." in value:
        value = value.rstrip("0").rstrip(".")
    return value


def format_indicator_value(value, *, force_integer=False, precise=False):
    if value is None:
        return "-"

    abs_value = abs(value)

    if abs_value > 999 and precise:
        nb = f"{round(value):,}".replace(",", " ")
    elif abs_value > 999999:
        nb = _remove_useless_0(_get_precision((value / 1_000_000), force_integer)) + "M"
    elif abs_value > 999:
        nb = _remove_useless_0(_get_precision((value / 1_000), force_integer)) + "k"
    else:
        nb = _remove_useless_0(_get_precision(value, force_integer))
    return nb.replace(".", ",")


def _format_value(k, v, *, precise=False):
    if k.lower().startswith("valeur"):
        v = format_indicator_value(v, precise=precise)
    return v


def format_data(data, *, precise=False):
    return {k: _format_value(k, v, precise=precise) for k, v in data.items()}



import csv

from django.http import HttpRequest, HttpResponse

from territories_dashboard_lib.tracking_lib.enums import EventType, GraphType
from territories_dashboard_lib.tracking_lib.logic import track_event

from .models import Indicator


def export_to_csv(
    request: HttpRequest, indicator: Indicator, graph_name: GraphType, data: list[dict]
):
    response = HttpResponse(content_type="text/csv")
    response["Content-Disposition"] = (
        f'attachment; filename="{indicator.name}_{graph_name}.csv"'
    )
    writer = csv.writer(response)
    if data:
        writer.writerow(data[0].keys())
    for row in data:
        writer.writerow(row.values())
    response = track_event(
        request=request,
        response=response,
        event_name=EventType.download,
        data={"indicator": indicator.name, "objet": graph_name, "type": "csv"},
    )
    return response



# Generated by Django 5.2.3 on 2025-06-19 14:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('website_lib', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='mainconf',
            name='contact_email',
            field=models.TextField(blank=True, default=None, null=True, verbose_name='Email de contact'),
        ),
        migrations.AddField(
            model_name='mainconf',
            name='newsletter_link',
            field=models.TextField(blank=True, default=None, null=True, verbose_name="Lien d'inscription à la newsletter"),
        ),
        migrations.AddField(
            model_name='mainconf',
            name='show_footer_contact_banner',
            field=models.BooleanField(default=False, help_text="Pour afficher la bannière, l'email de contact et le lien d'inscription à la newsletter doivent être renseignés.", verbose_name='Afficher une bannière de contact au-dessus du footer'),
        ),
    ]



# Generated by Django 5.2.3 on 2025-06-18 13:09

import martor.models
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='GlossaryItem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('word', models.TextField(verbose_name='Mot')),
                ('definition', models.TextField(verbose_name='Définition')),
            ],
            options={
                'verbose_name': 'Définition',
                'verbose_name_plural': 'Page Lexique',
                'ordering': ['word'],
            },
        ),
        migrations.CreateModel(
            name='LandingPage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('title', models.TextField(verbose_name='Titre')),
                ('body', martor.models.MartorField(verbose_name='Contenu')),
                ('button_link', models.TextField(help_text='Au format markdown, par exemple : <br/><br/>[découvrir les indicateurs](/indicateurs/)', verbose_name='Lien du bouton')),
            ],
            options={
                'verbose_name': "Page d'accueil",
                'verbose_name_plural': "Page d'accueil",
            },
        ),
        migrations.CreateModel(
            name='MainConf',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('title', models.TextField(default='Tableau de bord', verbose_name='Titre principal du site')),
                ('entity', models.TextField(default='', help_text="Renseigner le nom de l'entité au même format que le logo officiel en respectant les mises à la ligne.", verbose_name='Entité qui possède le site')),
                ('header_navigation', models.TextField(default='', help_text='Navigation du header, mettre les liens au format markdown avec un slash à la fin du lien, mettre une ligne vide entre chaque rubrique. <br/><br/>\n        \n        Une rubrique peut contenir un seul lien, ou bien plusiers, dans ce cas la rubrique sera sous forme de dropdown et il faut mettre un titre en premier. <br/><br/>Par exemple : <br/>\n        <br/><br/>\n        [Accueil](/accueil/)\n        <br/><br/>\n        [Indicateurs territoriaux](/indicateurs/)\n        <br/><br/>\n        À propos\n        [Présentation](/presentation/)\n        [Journal des versions](/journal/)\n        ', verbose_name='Navigation')),
                ('footer_navigation', models.TextField(default='', help_text='Navigation du footer, mettre les liens au format markdown, un par ligne. <br/><br/>\n    Si le lien est interne, commencer et terminer le lien par un slash comme "/accueil/"<br/><br/>\n    Par exemple : <br/><br/>\n    [Plan du site](/plan-site/)<br/>\n    [Accessibilité](/accessibilite/)<br/>\n    [Mentions légales](/mentions-legales/) <br/>\n    ', verbose_name='Navigation')),
            ],
            options={
                'verbose_name': 'Configuration principale',
                'verbose_name_plural': 'Configurations principales',
            },
        ),
        migrations.CreateModel(
            name='NoticeBanner',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('title', models.CharField(max_length=64, verbose_name='Titre')),
                ('description', models.CharField(max_length=255, verbose_name='Description')),
                ('link_href', models.CharField(blank=True, help_text="Le lien doit être un lien interne et commencer et terminer par '/', par exemple '/journal/", max_length=255, null=True, verbose_name='Lien')),
                ('link_text', models.CharField(blank=True, help_text='à remplir seulement si un lien est spécifié', max_length=64, null=True, verbose_name='Intitulé du lien')),
            ],
            options={
                'ordering': ['-created_at'],
            },
        ),
        migrations.CreateModel(
            name='StaticPage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.TextField(verbose_name='Nom de la page')),
                ('url', models.TextField(help_text="En minuscule avec des espaces remplacés par des traits '-'.<br/>Par exemple : <br/><br/> indicateurs <br/> mentions-legales", verbose_name='Url de la page')),
                ('body', martor.models.MartorField(verbose_name='Contenu')),
            ],
            options={
                'verbose_name': 'Page Statique',
                'verbose_name_plural': 'Pages Statiques',
            },
        ),
    ]
